@@ -7,7 +7,12 @@ Net::HTTP.start("uwsgi.it") do |http|
   end
 end
 
-system("bash install.sh rack #{Dir.pwd}/uwsgi.ruby")
+major,minor = RUBY_VERSION.split('.')
+if major.to_i >= 2 or (major.to_i == 1 and minor.to_i >= 9)
+  system("bash install.sh ruby2 #{Dir.pwd}/uwsgi.ruby")
+else
+  system("bash install.sh rack #{Dir.pwd}/uwsgi.ruby")
+end
 
 open("Makefile", "w") do |file|
   file.write("all:\n")
