@@ -48,12 +48,6 @@ struct uwsgi_option uwsgi_rack_options[] = {
         {"rbshell", optional_argument, 0, "run  a ruby/irb shell", uwsgi_opt_rbshell, NULL, 0},
         {"rbshell-oneshot", no_argument, 0, "set ruby/irb shell (one shot)", uwsgi_opt_rbshell, NULL, 0},
 
-#ifdef RUBY19
-        {"rb-threads", required_argument, 0, "set the number of ruby threads to run", uwsgi_opt_set_int, &ur.rb_threads, 0},
-        {"rbthreads", required_argument, 0, "set the number of ruby threads to run", uwsgi_opt_set_int, &ur.rb_threads, 0},
-        {"ruby-threads", required_argument, 0, "set the number of ruby threads to run", uwsgi_opt_set_int, &ur.rb_threads, 0},
-#endif
-
         {0, 0, 0, 0, 0, 0 ,0},
 
 };
@@ -861,6 +855,10 @@ int uwsgi_rack_request(struct wsgi_request *wsgi_req) {
 
 	rb_hash_aset(env, rb_str_new2("rack.errors"), rb_funcall( rb_const_get(rb_cObject, rb_intern("IO")), rb_intern("new"), 2, INT2NUM(2), rb_str_new("w",1) ));
 
+	rb_hash_aset(env, rb_str_new2("uwsgi.core"), INT2NUM(wsgi_req->async_id));
+	rb_hash_aset(env, rb_str_new2("uwsgi.version"), rb_str_new2(UWSGI_VERSION));
+	rb_hash_aset(env, rb_str_new2("uwsgi.node"), rb_str_new2(uwsgi.hostname));
+
 	// remove HTTP_CONTENT_LENGTH and HTTP_CONTENT_TYPE
 	rb_hash_delete(env, rb_str_new2("HTTP_CONTENT_LENGTH"));
 	rb_hash_delete(env, rb_str_new2("HTTP_CONTENT_TYPE"));
@@ -1240,11 +1238,22 @@ void uwsgi_ruby_init_thread(int core_id) {
 }
 
 void uwsgi_rack_postinit_apps(void) {
-
-	if (ur.rb_threads > 1) {
-	}
 }
 
+/*
+	If the ruby VM has rb_reserved_fd_p, we avoid closign the filedescriptor needed by
+	modern ruby (the Matz ones) releases.
+*/
+static void uwsgi_ruby_cleanup() {
+	int (*uptr_rb_reserved_fd_p)(int) = dlsym(RTLD_DEFAULT, "rb_reserved_fd_p");
+	if (!uptr_rb_reserved_fd_p) return;
+	int i;
+	for (i = 3; i < (int) uwsgi.max_fd; i++) {
+		if (uptr_rb_reserved_fd_p(i)) {
+			uwsgi_add_safe_fd(i);
+		}
+	}
+}
 
 struct uwsgi_plugin rack_plugin = {
 
@@ -1287,5 +1296,7 @@ struct uwsgi_plugin rack_plugin = {
 	.exception_repr = uwsgi_ruby_exception_repr,
 	.exception_log = uwsgi_ruby_exception_log,
 	.backtrace = uwsgi_ruby_backtrace,
+
+	.master_cleanup = uwsgi_ruby_cleanup,
 };
 
