@@ -9,14 +9,16 @@ static int uwsgi_routing_func_http(struct wsgi_request *wsgi_req, struct uwsgi_r
 	// mark a route request
         wsgi_req->via = UWSGI_VIA_ROUTE;
 
-	// get the http address from the route
-	char *addr = ur->data;
+	char **subject = (char **) (((char *)(wsgi_req))+ur->subject);
+        uint16_t *subject_len = (uint16_t *)  (((char *)(wsgi_req))+ur->subject_len);
+
+	struct uwsgi_buffer *ub_addr = uwsgi_routing_translate(wsgi_req, ur, *subject, *subject_len, ur->data, ur->data_len);
+	if (!ub_addr) return UWSGI_ROUTE_BREAK;
 
 	struct uwsgi_buffer *ub_url = NULL;
 	if (ur->data3_len) {
-		char **subject = (char **) (((char *)(wsgi_req))+ur->subject);
-        	uint16_t *subject_len = (uint16_t *)  (((char *)(wsgi_req))+ur->subject_len);
 		ub_url = uwsgi_routing_translate(wsgi_req, ur, *subject, *subject_len, ur->data3, ur->data3_len);
+		uwsgi_buffer_destroy(ub_addr);
         	if (!ub_url) return UWSGI_ROUTE_BREAK;
 	}
 
@@ -25,7 +27,8 @@ static int uwsgi_routing_func_http(struct wsgi_request *wsgi_req, struct uwsgi_r
 	struct uwsgi_buffer *ub = uwsgi_to_http(wsgi_req, ur->data2, ur->data2_len, ub_url ? ub_url->buf : NULL, ub_url ? ub_url->pos : 0);	
 	if (!ub) {
 		if (ub_url) uwsgi_buffer_destroy(ub_url);
-		uwsgi_log("unable to generate http request for %s\n", addr);
+		uwsgi_log("unable to generate http request for %s\n", ub_addr->buf);
+		uwsgi_buffer_destroy(ub_addr);
                 return UWSGI_ROUTE_NEXT;
 	}
 
@@ -37,7 +40,8 @@ static int uwsgi_routing_func_http(struct wsgi_request *wsgi_req, struct uwsgi_r
 	if (wsgi_req->proto_parser_remains > 0) {
 		if (uwsgi_buffer_append(ub, wsgi_req->proto_parser_remains_buf, wsgi_req->proto_parser_remains)) {
 			uwsgi_buffer_destroy(ub);
-			uwsgi_log("unable to generate http request for %s\n", addr);
+			uwsgi_log("unable to generate http request for %s\n", ub_addr->buf);
+			uwsgi_buffer_destroy(ub_addr);
                		return UWSGI_ROUTE_NEXT;
 		}
 		wsgi_req->proto_parser_remains = 0;
@@ -49,22 +53,25 @@ static int uwsgi_routing_func_http(struct wsgi_request *wsgi_req, struct uwsgi_r
 		if (uwsgi.post_buffering > 0 && wsgi_req->post_cl > 0) {
 			if (uwsgi_buffer_append(ub, wsgi_req->post_buffering_buf, wsgi_req->post_cl)) {
 				uwsgi_buffer_destroy(ub);
-				uwsgi_log("unable to generate http request for %s\n", addr);
+				uwsgi_log("unable to generate http request for %s\n", ub_addr->buf);
+				uwsgi_buffer_destroy(ub_addr);
                			return UWSGI_ROUTE_NEXT;
 			}
 		}
-        	if (!uwsgi_offload_request_net_do(wsgi_req, addr, ub)) {
+        	if (!uwsgi_offload_request_net_do(wsgi_req, ub_addr->buf, ub)) {
                 	wsgi_req->via = UWSGI_VIA_OFFLOAD;
 			wsgi_req->status = 202;
+			uwsgi_buffer_destroy(ub_addr);
 			return UWSGI_ROUTE_BREAK;
                 }
 	}
 
-	if (uwsgi_proxy_nb(wsgi_req, addr, ub, remains, uwsgi.shared->options[UWSGI_OPTION_SOCKET_TIMEOUT])) {
-		uwsgi_log("error routing request to http server %s\n", addr);
+	if (uwsgi_proxy_nb(wsgi_req, ub_addr->buf, ub, remains, uwsgi.shared->options[UWSGI_OPTION_SOCKET_TIMEOUT])) {
+		uwsgi_log("error routing request to http server %s\n", ub_addr->buf);
 	}
 
 	uwsgi_buffer_destroy(ub);
+	uwsgi_buffer_destroy(ub_addr);
 
 	return UWSGI_ROUTE_BREAK;
 
